﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Configuration;
using Microsoft.PowerBI.Api.V2;
using Microsoft.PowerBI.Api.V2.Models;
using Microsoft.Rest;
using Newtonsoft.Json;

namespace Sales.API.Controllers
{
    [Route("api/[controller]")]
    [ApiController]
    public class PowerBIController : Controller
    {
        private static bool configInitialized = false;
        private static string AuthorityUrl;
        private static string ResourceUrl;
        private static string ApplicationId;
        private static string ApplicationSecret;
        private static string ApiUrl;
        private static string WorkspaceId;
        private static string ReportId;
        private static string DatasetId;

        private static string MasterUsername;
        private static string MasterPassword;

        readonly IConfiguration _config;
        public PowerBIController(IConfiguration config)
        {
            this._config = config;
            InitConfigValues();
        }

        private void InitConfigValues()
        {
            if (!configInitialized)
            {
                AuthorityUrl = _config.GetSection("PowerBI").GetValue<string>("AuthorityUrl");
                ResourceUrl = _config.GetSection("PowerBI").GetValue<string>("ResourceUrl");
                ApplicationId = _config.GetSection("PowerBI").GetValue<string>("ApplicationId");
                ApplicationSecret = _config.GetSection("PowerBI").GetValue<string>("ApplicationSecret");
                ApiUrl = _config.GetSection("PowerBI").GetValue<string>("ApiUrl");
                WorkspaceId = _config.GetSection("PowerBI").GetValue<string>("WorkspaceId");
                ReportId = _config.GetSection("PowerBI").GetValue<string>("ReportId");
                DatasetId = _config.GetSection("PowerBI").GetValue<string>("DatasetId");
                MasterUsername = _config.GetSection("PowerBI").GetValue<string>("MasterUserId");
                MasterPassword = _config.GetSection("PowerBI").GetValue<string>("MasterUserPassword");
            }
        }

        [HttpGet("EmbedConfig")]
        public EmbedConfig EmbedReport()
        {
            EmbedConfig report = null;
            var proUserToken = AuthenticateAsync().GetAwaiter().GetResult();
            List<EffectiveIdentity> filters = new List<EffectiveIdentity>();
            filters.Add(new EffectiveIdentity
            {
                Username = Request.Query["userId"],
                Roles = new List<string> { "Permissions" },
                Datasets = new List<string> { DatasetId }
            });
            report = GenerateReport(proUserToken.AccessToken, filters);
            return report;
        }

        private async Task<OAuthResult> AuthenticateAsync()
        {
            var oauthEndpoint = new Uri(AuthorityUrl);

            using (var client = new HttpClient())
            {
                var result = await client.PostAsync(oauthEndpoint, new FormUrlEncodedContent(new[]
                    {
                    new KeyValuePair<string,string>("resource", ResourceUrl),
                    new KeyValuePair<string,string>("client_id", ApplicationId),
                    new KeyValuePair<string,string>("client_secret", ApplicationSecret),
                    new KeyValuePair<string,string>("grant_type", "password"),
                    new KeyValuePair<string,string>("username", MasterUsername),
                    new KeyValuePair<string,string>("password", MasterPassword),
                    new KeyValuePair<string,string>("scope", "openid")
                }
                ));

                var content = await result.Content.ReadAsStringAsync();
                return JsonConvert.DeserializeObject<OAuthResult>(content);
            }
        }

        private EmbedConfig GenerateReport(string token, List<EffectiveIdentity> filters = null)
        {
            EmbedConfig config = null;
            var tokenCredentials = new TokenCredentials(token, "Bearer");

            // Create a Power BI Client object. It will be used to call Power BI APIs.
            using (var client = new PowerBIClient(new Uri("https://api.powerbi.com"), tokenCredentials))
            {
                Report report = client.Reports.GetReportInGroup(WorkspaceId, ReportId);
                if (report != null)
                {
                    var requestParameters = new GenerateTokenRequest();
                    requestParameters.AccessLevel = "View";
                    if (filters != null)
                    {
                        requestParameters.Identities = filters;
                    }

                    // Generate EmbedToken This function sends the POST message 
                    //with all parameters and returns the token
                    EmbedToken embedtoken = client.Reports.GenerateTokenInGroupAsync(
                        WorkspaceId,
                        ReportId,
                        requestParameters).GetAwaiter().GetResult();

                    config = new EmbedConfig();
                    config.EmbedURL = report.EmbedUrl;
                    config.GroupID = WorkspaceId;
                    config.ReportData = report;
                    config.ReportID = ReportId;
                    config.Token = embedtoken?.Token;
                    config.TokenID = embedtoken?.TokenId;
                    config.Expiration = embedtoken?.Expiration;
                }

            }
            return config;
        }


    }

    public class OAuthResult
    {
        [JsonProperty("token_type")]
        public string TokenType { get; set; }
        [JsonProperty("scope")]
        public string Scope { get; set; }
        [JsonProperty("expires_in")]
        public int ExpiresIn { get; set; }
        [JsonProperty("ext_expires_in")]
        public int ExtExpiresIn { get; set; }
        [JsonProperty("expires_on")]
        public int ExpiresOn { get; set; }
        [JsonProperty("not_before")]
        public int NotBefore { get; set; }
        [JsonProperty("resource")]
        public Uri Resource { get; set; }
        [JsonProperty("access_token")]
        public string AccessToken { get; set; }
        [JsonProperty("refresh_token")]
        public string RefreshToken { get; set; }
    }

    public class EmbedConfig
    {
        public string EmbedURL { get; set; }
        public string ReportID { get; set; }
        public string GroupID { get; set; }
        public Report ReportData { get; set; }
        public string Token { get; set; }
        public string TokenID { get; set; }
        public DateTime? Expiration { get; set; }
    }
}
